function [noisy_spike_train,tt_train, valid_firing_onsets] = spike_generator(varargin)
%[noisy_spike_train,tt_train, valid_firing_onsets] = spike_generator(samp_rate, spike_duration, train_duration, spike_abscissas, spike_ordinates, av_fir_rate, std_fir_rate, SNRdB, show)
%
%   This function generates spike trains based on the repetition of spike
%   waveforms described through linear interpolatin between points.
%
%   INPUT:
%   samp_rate       = sampling rate {samples/s} [1 x 1] (default = 20000);
%   spike_duration  = temporal support of the single spike waveform {s} [1 x 1] (default = 3E-3 s);
%   train_duration  = temporal support of the spike train {s} [1 x 1] (default = 10 s);
%   spike_abscissas = abscissas of points describing the spike waveform {s} [] (default = [0.1E-3, 0.9E-3, 1.3E-3, 1.8E-3] s);
%   spike_ordinates = ordinates of points describing the spike waveform {V} [] (default = [0, 50E-3, -30E-3, 0] V);
%   av_fir_rate     = average value of the spike firing rate {Hz} [1 x 1] (default = 35 Hz);
%   std_fir_rate    = std dev value of the spike firing rate {Hz} [1 x 1] (default = 10 Hz);
%   SNRdB           = SNR value requested  {dB} [1 x 1] (default = 18 dB);
%   show            = binary variable (1 to show behaviour, no show otherwise)
%
%   Recommendation 1: omit or leave empty brackets for default
%
%   Recommendation 2: spike_abscissas and spike_ordinates need to be
%   consistent (i.e. same length)
%
%   OUTPUT:
%   noisy_spike_train   = time series of spike_train [(train_duration x samp_rate) x 1];
%   tt_train            = temporal axis [(train_duration x samp_rate) x 1];
%   valid_firing_onsets = temporal locations of firing onsets;
%
%   DEPENDENCIES:
%   none
%
%
%   This routine was written by:
%   Maurizio Schmid, PhD <mailto: schmid@uniroma3.it>
%   BioLab, Dept Applied Electronics, Roma Tre University, Rome, Italy
%
%   Last Updated: Apr 6, 2011
%   Please retain the author names, and give acknowledgement where necessary.
%   The author does not assume any responsibility related to the use
%   of this code, and does not guarantee its correctness.

%% Phase 0. Init costant/variables

%default values are used if no value is reported as input
default_binary = ones(1, 9);
for pp = 1:length(varargin)
    if ~isempty(varargin{pp})
        default_binary(pp) = 0;
    end
end


if default_binary(9)  show = 0; else show = varargin{9}; end
if default_binary(8)  SNRdB = 18; else SNRdB = varargin{8}; end
if default_binary(7)  std_fir_rate = 10; else std_fir_rate = varargin{7}; end
if default_binary(6)  av_fir_rate = 35; else av_fir_rate = varargin{6}; end
if default_binary(5)  spike_ordinates = [0, 50E-3, -30E-3, 0]; else spike_ordinates = varargin{5}; end
if default_binary(4)  spike_abscissas = [0.1E-3, 0.9E-3, 1.3E-3, 1.8E-3]; else spike_abscissas = varargin{4}; end
if default_binary(3)  train_duration = 10; else train_duration = varargin{3}; end
if default_binary(2)  spike_duration = 3E-3; else spike_duration = varargin{2}; end
if default_binary(1)  samp_rate = 20000; else samp_rate = varargin{1}; end

%% Phase 1. Generate single spike waveform
%1.1 init working variables
tt_spike = 0 : 1/samp_rate : spike_duration;
spike = zeros(1,length(tt_spike));
no_segments = length(spike_abscissas) - 1;
m = zeros(1, no_segments);
q = zeros(1, no_segments);
index_initial_pos = zeros(1, no_segments);
index_final_pos = zeros(1, no_segments);

%1.1. Compute angular coefficients and intercept for each linear segment
for jj = 1 : no_segments
    m(jj) = (spike_ordinates(jj + 1) - spike_ordinates(jj))/(spike_abscissas(jj + 1) - spike_abscissas(jj));
    q(jj) = spike_ordinates(jj) - m(jj)*spike_abscissas(jj);
    index_initial_pos(jj) = find(abs(tt_spike-spike_abscissas(jj)) == min(abs(tt_spike-spike_abscissas(jj))));
    index_final_pos(jj) = find(abs(tt_spike-spike_abscissas(jj+1)) == min(abs(tt_spike-spike_abscissas(jj+1))));
    spike(index_initial_pos(jj):index_final_pos(jj)) = m(jj)*tt_spike(index_initial_pos(jj):index_final_pos(jj)) + q(jj);
end

%1.2. Display results
if show
    figure
    subplot(2,2,1), plot(tt_spike, spike, 'LineWidth', 3);
    title('Single spike');
    xlabel('s')
    ylabel('V')
end


%% 2. Generate spike train
%2.1. Generate firing rates realizations
max_num_spikes = train_duration * (av_fir_rate + (4 * std_fir_rate));
firing_rates = av_fir_rate + std_fir_rate * randn(1,max_num_spikes);
firing_onsets = zeros(1,max_num_spikes);
tt_train = 0 : 1/samp_rate : train_duration;
spike_train = zeros(1,length(tt_train));

%2.2. Display results
if show
    subplot(2,2,2), hist(firing_rates);
    title('Firing rate histogram');
    xlabel('spikes/s')
end

%2.3. Compute firing_onsets
ipi = 1./firing_rates;
for jj = 1 : length(ipi)
    firing_onsets(jj) = sum(ipi(1:jj));
end

%2.4. Toss indexes that occur after the train duration
valid_indexes = firing_onsets < train_duration - spike_duration;
valid_firing_onsets = firing_onsets(valid_indexes);


%2.5. Insert Spikes at computed firing occurrences
for jj = 1 : length(valid_firing_onsets)
    spike_train(round(valid_firing_onsets(jj) * samp_rate) + 1 : round(valid_firing_onsets(jj) * samp_rate) + length(spike)) = spike;
end


%% 3. Add noise
%3.1. Compute noise power
pow_spike = var(spike) + mean(spike).^2;

pow_noise = pow_spike / (10^(SNRdB/10));
std_noise = sqrt(pow_noise);
noisy_spike_train = spike_train + std_noise*randn(1,length(spike_train));

%3.2. Display results
if show
    subplot(2,2,[3,4]), plot(tt_train,noisy_spike_train), hold on
    plot(valid_firing_onsets + spike_abscissas(2),noisy_spike_train(round((valid_firing_onsets + spike_abscissas(2)) * samp_rate)),'ok')
    title('Noisy Spike Train')
    xlabel('s')
    ylabel('V')
    legend('spike train','firing occurrences')
end
